/*
* You can find  the original version here:
* http://www.gnu.org/software/grub/manual/multiboot/multiboot.html
*/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
/* Macros. */
     
/* The magic number for the Multiboot header. */
.set MULTIBOOT_HEADER_MAGIC ,         0x1BADB002

/* The flags for the Multiboot header. */
.set MULTIBOOT_HEADER_FLAGS ,        0x00000003

/* The size of our stack (16KB). */
.set STACK_SIZE,                      0x4000
      
.text

.globl  start, _start

start:
_start:
			 jmp     multiboot_entry

			 /* Align 32 bits boundary. */
			 .align  4

/* Multiboot header. */
multiboot_header:

			 /* magic */
			 .long   MULTIBOOT_HEADER_MAGIC
			 /* flags */
			 .long   MULTIBOOT_HEADER_FLAGS
			 /* checksum */
			 .long   -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
			 
multiboot_entry:
			 /* Initialize the stack pointer. */
			 movl    $(stack + STACK_SIZE), %esp

			 /* Reset EFLAGS. */
			 pushl   $0
			 popf

			 /* Push the pointer to the Multiboot information structure. */
			 pushl   %ebx
			 /* Push the magic value. */
			 pushl   %eax

			 /* Now enter the C main function... */
			 call   kmain

/* Halt. */		 
loop:   hlt
			 jmp     loop

/* Our stack area. */
.comm   stack, STACK_SIZE
