/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
#include <console.h>
#include <memory.h>
#include <floppy.h>
#include <fat.h>

tp_fat_boot boot;


unsigned int fat_region;
unsigned int root_region;
unsigned int data_region;

unsigned char fd_buffer[BLOCK_SIZE];

void read_boot()
{
	
		read_block(0,fd_buffer);
	
		memcpy((void *)&boot,fd_buffer,sizeof(tp_fat_boot));
	
		// In blocks
		fat_region = boot.reserved_sec*(boot.bps/BLOCK_SIZE);
		root_region = fat_region + (boot.fat_num*boot.spf)*(boot.bps/BLOCK_SIZE);
		data_region = root_region + ((boot.root_entries * 32) / boot.bps)*(boot.bps/BLOCK_SIZE);
}

void ls_root()
{
	tp_fat_dir root;
	
	int i,j;
	char name[10];
	
	read_block(root_region,fd_buffer);
	
	for (i=0;; i++)
	{
		memcpy((void *)&root,&fd_buffer[i*0x20],sizeof(tp_fat_dir));
		
		if (root.file_name[0]==0)
			break;
		else 
		// Volume name is not set 
		if (!(root.attrib & 0x08)) 
			// If the first byte is E5 the file it's delete (hide them)
			if (root.file_name[0]!=0xE5)
			{
				for(j=0; j<8 && root.file_name[j]!=' '; j++)
					name[j]=root.file_name[j];
				name[j]='\n';
				name[j+1]='\0';
				put_string(name);
			}	
	}
}

// Find the next cluster in fat (next in file chain)
unsigned short int fat_next_cluster (unsigned short int cluster)
{
	int offset_fat_entry; // In bits
	
	char next_cluster_L;
	char next_cluster_H;

	// In fat12 each entry is 12bits
	offset_fat_entry = cluster * 12;
	
	// Read the low part of next_cluster
	read_block(fat_region + (offset_fat_entry/BLOCK_SIZE),fd_buffer);
	next_cluster_L = fd_buffer[(offset_fat_entry%BLOCK_SIZE)/8];
	
	// Read the high part of next_cluster
	// if it's the last byte re-read the next block
	if ((offset_fat_entry%BLOCK_SIZE)/8 == 512) 
		read_block(fat_region + ((offset_fat_entry+8)/BLOCK_SIZE),fd_buffer);		
	next_cluster_H = fd_buffer[((offset_fat_entry+8)%BLOCK_SIZE)/8];
	
	if ((offset_fat_entry%BLOCK_SIZE)%8 == 0)
		next_cluster_H = next_cluster_H & 0x0F;  // Get the low part
	else
		next_cluster_L = next_cluster_L << 4;  // Get the high part
	
	return next_cluster_L + (next_cluster_H*0x0F);
}

// Read num_bytes from file or directory
char read(unsigned short int cluster,int offset,char * read_buffer, int num_bytes)
{
	int relative_cluster;
	int relative_block;
	int relative_offset;
	
	int absolute_block;
	
	unsigned short int new_cluster;
	
	int rest_bytes;
	
	int i;
	
	// Number of clusters in the chain before of the byte cluster
	relative_cluster = offset/(boot.bps*boot.spc);
	// Number of blocks before of the byte in the cluster 
	relative_block = (offset / BLOCK_SIZE) % ((boot.bps*boot.spc)/BLOCK_SIZE);
	// Number of bytes before of the byte in the block
	relative_offset = offset - relative_cluster*(boot.bps*boot.spc) \
										- relative_block * BLOCK_SIZE;
	
	new_cluster = cluster;
	rest_bytes = num_bytes;
	
	
	// Find the first cluster to read 
	for (i=0; i<relative_cluster; i++)
	{
		if (new_cluster>=0xFF8)
			return -1; // Offset out of end of file 
		
		// Find the next cluster
		new_cluster = fat_next_cluster(new_cluster);	
	}
	
	
	new_cluster -= 2; // First an second clusters unused
	
	// Absolute number of block
	absolute_block = new_cluster*((boot.bps*boot.spc)/BLOCK_SIZE) + relative_block;
	
	read_block(data_region + absolute_block,fd_buffer);
	
	// If requested bytes are in more than one block , read the complete blocks
	while ((relative_offset+rest_bytes)>BLOCK_SIZE) 
	{
		// Copy the actual block to read_buffer
		memcpy(&read_buffer[num_bytes-rest_bytes],&fd_buffer[relative_offset],BLOCK_SIZE-relative_offset);
		
		relative_offset = 0;
		rest_bytes -= BLOCK_SIZE-relative_offset;
		
		// Check if remain blocks in this cluster
		if (((boot.bps*boot.spc)/BLOCK_SIZE)>relative_block+1)
		{
			// No remain blocks in this cluster, read another one
			
			// Next cluster to read
			new_cluster = fat_next_cluster(new_cluster + 2) - 2;
			
			if ((new_cluster+2)>=0xFF8)
				return -1; // Offset out of end of file 
			
			relative_block = 0;
			absolute_block = new_cluster*((boot.bps*boot.spc)/BLOCK_SIZE);
		}else
			absolute_block++; // Read the next block in cluster
		
		// Read the block
		read_block(data_region + absolute_block,fd_buffer);
	}
	
	// copy the rest of the last block 
	memcpy(&read_buffer[num_bytes-rest_bytes],&fd_buffer[relative_offset],rest_bytes);
	
	return 0;
	
}
