/*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU Library General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <gdt.h>
#include <console.h>
 
int init_gdt()
{
	int i;
	tp_gdt_entry gdt_entry;
	tp_gdt_entry gdt_null_entry;
	
	gdt_null_entry.seg_limit_l = 0;
	gdt_null_entry.base_l = 0;
	gdt_null_entry.base_m = 0;
	gdt_null_entry.type = 0;
	gdt_null_entry.seg_type = 0;
	gdt_null_entry.p_level = 0;
	gdt_null_entry.present = 0;
	gdt_null_entry.seg_limit_h = 0;
	gdt_null_entry.attrib = 0;
	gdt_null_entry.base_h = 0;
	
	load_gdt_entry(&gdt_null_entry,0); // The first descriptor is always null
	 
	gdt_entry.seg_limit_l = 0x0FFF; // 4K Pages (4K*4K=16MB)
	gdt_entry.base_l = 0x0000;
	gdt_entry.base_m = 0x00;
	gdt_entry.type = GDT_COD_ER;
	gdt_entry.seg_type = GDT_CODE_DATA;
	gdt_entry.p_level = GDT_RING_0;
	gdt_entry.present = GDT_PRESENT;
	gdt_entry.seg_limit_h = 0x0;
	gdt_entry.attrib = GDT_SIZE_32|GDT_GRAN_PAGES;
	gdt_entry.base_h = 0x00;	
	
	load_gdt_entry(&gdt_entry,1); 
	/* 
	* cs=0x08=00001000b because in PAGE mode the system
	*  use bits 3-15 to index de gdt table.
	*  The next entry cs=0x10=00010000b ... 
	*/
	
	for(i=2; i<256; i++)
		load_gdt_entry(&gdt_null_entry,i); // Others are empty
	
	load_gdtr();
	
	put_string("GDT Initialization [ DONE ]\n");
	
	return 1;
	
}
 
// Load de new GDT entry
void load_gdt_entry(tp_gdt_entry *gdt_entry, int gdt_num)
{
	tp_gdt_entry * new_gdt_entry;
	// new_gdt_entry points to the gdt_entry estuct in memory
	new_gdt_entry = (tp_gdt_entry*) GDT_BASE + gdt_num;
	
	// Copy the descriptor to the GDT memory 
	// TODO: replace with memcpy when exists
	new_gdt_entry->seg_limit_l = gdt_entry->seg_limit_l;
	new_gdt_entry->base_l = gdt_entry->base_l;
	new_gdt_entry->base_m = gdt_entry->base_m;
	new_gdt_entry->type = gdt_entry->type;
	new_gdt_entry->seg_type = gdt_entry->seg_type;
	new_gdt_entry->p_level = gdt_entry->p_level;
	new_gdt_entry->present = gdt_entry->present;
	new_gdt_entry->seg_limit_h = gdt_entry->seg_limit_h;
	new_gdt_entry->attrib = gdt_entry->attrib;
	new_gdt_entry->base_h = gdt_entry->base_h;
	
}

// Load de GDT register
void load_gdtr()
{
	tp_gdtr gdtr;
	
	// Fill GDT register struct
	gdtr.length = (unsigned short int) (8*256);
	gdtr.base_l = (unsigned short int) GDT_BASE_L;
	gdtr.base_h = (unsigned short int) GDT_BASE_H;

	// Load GDT register 
	asm("lgdt (%0)": :"p" (&gdtr));
}
