/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <idt.h>
#include <console.h>

int idt_limit = 0; //Max interrupt registered

void init_idt()
{
	/* TODO: load all exceptions and then call load_idtr 
		note: This function must be called after load all idt_entries */
	
	load_idtr();
	
	put_string("IDT Initialization [ DONE ]\n");
}

void load_idt_entry(int idt_num, unsigned int offset,
	unsigned int selector, unsigned char options)
{
	unsigned int offset_h, offset_l;
	tp_idt_entry * new_idt_entry; 
	
	idt_limit = idt_limit>idt_num?idt_limit:idt_num;
	// new_idt_entry points to the idt_entry estuct in memory
	new_idt_entry = (tp_idt_entry *) IDT_BASE + idt_num;

	// Divide offset var in high and low
	offset_l = offset & 0x0000FFFF;
	offset_h = (offset & 0xFFFF0000) >> 16;
	
	// Copy the descriptor to the IDT memory 	
	new_idt_entry->offset_l = offset_l;	
	new_idt_entry->offset_h = offset_h;	
	new_idt_entry->selector = selector;			
	new_idt_entry->options = options;
}

// Load de IDT register 
void load_idtr()
{
	/*This function must be called after load all idt_entries
		else the nexts entries will be ignored. */
	
	tp_idtr idtr;	
	
	// Fill IDT register struct
	idtr.limit = (unsigned short int) (8*(idt_limit+1));
	idtr.base_l = (unsigned short int) IDT_BASE_L;
	idtr.base_h = (unsigned short int) IDT_BASE_H;

	// Load IDT register 
	asm("lidt (%0)": :"p" (&idtr));
}
