/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef __fat_h__
#define __fat_h__

#define BLOCK_SIZE 512 // Size block of the floppy driver

// FAT12 boot struct
typedef struct fat_boot_struct
{
  unsigned char jump[3]; // Jump 
  unsigned char os_name[8]; // OEM ID or formating os name
  unsigned short int bps; // Bytes per sector
  unsigned char spc; // Sectors per cluster
  unsigned short int reserved_sec;  //Reserved sectors
  unsigned char fat_num; // Number of FAT's
  unsigned short int root_entries; // Number of posible root entries 
  unsigned short int total_sec; // Total sectors
  unsigned char media_desc; // Media descriptor (1.44 Mb ...)
  unsigned short int spf; // Sectors per FAT 
  unsigned short int spt; // Sectors per Track
  unsigned short int heads; // Number of heads
  unsigned short int hidden_sec; // Hidden sectors
} __attribute__ ((packed)) tp_fat_boot; // Else the struct fields are wide to align


// FAT12 dir struct
typedef struct fat_dir_struct
{
	unsigned char file_name[8]; // File name
	unsigned char extension[3]; // Extension
	unsigned char attrib; // Atributes
	unsigned short int reserved; // Reserved
	unsigned short int time; // Time
	unsigned short int date; // Date
	unsigned short int entry_cluster; // Entry cluster (block) value
	unsigned short int file_size; // File size (Bytes)
} __attribute__ ((packed))tp_fat_dir; // Else the struct fields are wide to align

void read_boot();
void ls_root();
char read(unsigned short int cluster,int offset,char * read_buffer, int num_bytes);
#endif
