/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <idt.h>
#include <io.h>
#include <pic.h>
#include <console.h>
#include <keyboard.h>

	
char key_map[] ={'0', '0', '1' , '2' , '3' , '4' , '5' , '6' , '7' , '8' , '9',  
	'0' , '-' , '=' , '0' , '0',   'q' , 'w' , 'e' , 'r' , 't' , 'y' , 
	'u' , 'i' , 'o' , 'p' , '[' , ']' , '0', '0', 
	'a' , 's' , 'd' , 'f' , 'g' , 'h' , 'j' , 'k' , 'l' , 
	';' , 0x27 , '`' , '0' , 0x5C, 'z' , 'x' , 'c' , 'v' , 'b' , 
	'n' , 'm' , ',' , '.' , '/' }; 	
	
char shift_state = 0;

void init_keyboard ()
{
	// Load the IDT keyboard handler 
	load_idt_entry(33,(unsigned int)&keyboard_handler,IDT_CS_SELECTOR,
	IDT_INT_GATE|IDT_SIZE_32|IDT_RING_0|IDT_PRESENT);
	
	// Enable IRQ of keyboard
	enable_irq(KEYBOARD_IRQ);

	
	put_string("Keyboard Initialized [ DONE ]\n");
}


void keyboard_handler()
{

unsigned char key;

char ch;

key = inb(0x60);
	
switch(key & 0x7F)
{	
	case 0x2A:  // Shift left
	case 0x36:	// Shift right
		shift_state = ~shift_state; // XOR, invert the state
		break;
	
	case 0x3A:	// Lock caps
		// Detect only when it's pressed, not when it's released
		if (key == 0x3A)
			shift_state = ~shift_state; // XOR, invert the state
		break;
		
	case 0x0E: // Backspace
		// Detect only when it's pressed, not when it's released
		if (key == 0x0E)
			del_char();
		break;
		
	case 0x1C: // Return
		// Detect only when it's pressed, not when it's released
		if (key == 0x1C)
			new_line();
		break;
		
	default:
		if (!(key&0x80))
		{
			ch = key_map[key];
			
			if (shift_state && ch >= 'a' && ch <= 'z')
				ch += ('A'-'a');
	
			put_char(ch);
		}
		
}		
	
// Devolvemos el control de la int
outb(0x20, 0x20);
	
asm("mov %ebp,%esp");
asm("pop %ebp");
asm("iret");

}
